using UnityEngine;
using System.Collections;
using Leap;

// Draws the fingers onto the screen.
public class LeapFingerDraw : MonoBehaviour {
	Controller m_leapController;
	Texture2D m_noTouch;
	Texture2D m_touch;
	Texture2D m_borderTexture;
	
	void Start() {
		m_leapController = new Controller();
		m_noTouch = Resources.Load("Texture/FingerCircle") as Texture2D;
		m_touch = Resources.Load("Texture/FingerCircleTouch") as Texture2D;
		m_borderTexture = Resources.Load("Texture/FingerCircleOutline") as Texture2D;
	}
	
	// visual feedback for finger positions
	void OnGUI() {
		Frame frame = m_leapController.Frame();
		InteractionBox interactionBox = frame.InteractionBox;
		for(int i = 0; i < frame.Pointables.Count; ++i) {
			Vector3 screenPos = interactionBox.NormalizePoint(frame.Pointables[i].StabilizedTipPosition).ToUnity();
			float x = screenPos.x * UnityEngine.Screen.width;
			// y is flipped in GUI space.
			float y = (1 - screenPos.y) * UnityEngine.Screen.height;
			
			// compute the size scalar of the finger circle.
			float touchScalar = Mathf.Clamp(frame.Pointables[i].TouchDistance, -0.05f, frame.Pointables[i].TouchDistance);
			float size = 100.0f * Mathf.Sqrt(Mathf.Min(Mathf.Max(Mathf.Abs(touchScalar), 0.001f), 0.9f));

			Vector2 normXY = new Vector2(frame.Pointables[i].TipVelocity.x, frame.Pointables[i].TipVelocity.y);
			
			// non uniform scaling for ellipsoid effect when finger is moving.
			float yScale = 1.0f + Mathf.Min(0.6f, normXY.magnitude / 2000.0f);
			float xScale = Mathf.Sqrt(1.0f / yScale);
			
			Matrix4x4 mat = GUI.matrix;
			
			float angle = Mathf.Rad2Deg * Mathf.Atan(normXY.x / normXY.y);
			
			// rotate the ellipsoid based on direction of the finger velocity.
			GUIUtility.RotateAroundPivot(angle, new Vector2(x, y));
			if (frame.Pointables[i].TouchDistance > 0) {
				float outerSizeX = ((size + 10) * xScale);
				float outerSizeY = ((size + 10) * yScale);
				GUI.DrawTexture(new Rect(x - outerSizeX / 2, y - outerSizeY / 2, outerSizeX, outerSizeY), m_borderTexture, ScaleMode.StretchToFill, true);
				float innerSizeX = (size * xScale);
				float innerSizeY = (size * yScale);
				GUI.DrawTexture(new Rect(x - innerSizeX / 2, y - innerSizeY / 2, innerSizeX, innerSizeY), m_noTouch, ScaleMode.StretchToFill, true);
			} else {
				float scaleX = (size * xScale + 10);
				float scaleY = (size * yScale + 10);
				GUI.DrawTexture(new Rect(x - scaleX / 2, y - scaleY / 2, scaleX, scaleY), m_touch, ScaleMode.StretchToFill, true);
			}
			
			
			GUI.matrix = mat;
		}
	}
}
